#!/opt/bin/perl -w

#                             $cell_1
#                                |
#                       -------------------
#                       |                 |
#                    $cell_3[0]         $cell
#                       |                 |
#               --------------        --------------
#               |            |        |            |
#           cell_4[0]    $cell_4[1]  $cell_2[0]   $cell_2[1]




use lib '..','../..';
use strict;

use File::stat;
# use URI::Escape;
use CGI qw/:standard :html3 escape Map area *dl *blockquote/;
use CGI::Carp qw/fatalsToBrowser/;
use Ace 1.51;
use Ace::Sequence;
use ElegansSubs;
use AceSubs;

my $cell = param('name');
if (!$cell) {
    Error();
    exit 0;
}
# start Ace
my $DB = OpenDatabase() || AceError("Couldn't open database.");

#collect pedigree informations
my $results  = $DB->fetch(-class=>'Cell',
			  -name=>$cell);
# error
if (!defined($results)){
    Error($cell);
    exit 0;
}

#logo and yellow bar
my $class = "Cell";
print_top($cell,$class);  

#print the pictures (call to make_pedigree.cgi).
my ($cell_1,$cell_2) = Pedigree_map($results) if defined($results);
my $link_diagram = $results->get('Link_diagram'=>1);

#print search bar
print_prompt_hunter();

#now starts the report 

####remark
my $remark= $results->get('Remark'=>1);

if (defined($remark)){
  print img({-src=>'/images/cross1.gif'}),b(' ABSTRACT: '),$remark,br,br;
}else{
    print img({-src=>'/images/cross1.gif'}),b(' ABSTRACT: '),' No description for this cell.',br,br;
}

####Cell group

my $cell_group= $results->get('Cell_group'=>1);

if (defined($cell_group)){
    my $results  = $DB->fetch(-class=>'Cell_group',
			  -name=>$cell_group);
    my $value= $results->Remark(1);
    print img({-src=>'/images/cross1.gif'}),b(' GROUP: '),' Part of ',a({-href=>Object2URL($cell_group)},$value), 
    ' cell group.',br,br;
}else{
    print img({-src=>'/images/cross1.gif'}),b(' GROUP: '),' No group defined',br,br;
}


####Fate

my $fate= $results->get('Fate'=>1);

if (defined($fate)){
    print img({-src=>'/images/cross1.gif'}),b(' FATE: '),' The final destination of this particular groups of cells is to compose ',$fate, ' cells.',br,br;
}else{
    print img({-src=>'/images/cross1.gif'}),b(' FATE: '),' Value not present.',br,br;
}


##pedigree informations
if (defined($cell_1)){
    print img({-src=>'/images/cross1.gif'}),b(' PEDIGREE: '),
    ' It is derived from the cell named ',ObjectLink($cell_1);
  
  
    if ((defined($cell_2->[0]))&&(defined($cell_2->[1]))){
	print '; it is the precursor of the the cell named ',ObjectLink($cell_2->[0]), 
	' and the cell named ',ObjectLink($cell_2->[1]),'.';
    }
  
    if ((!defined($cell_2->[0]))&&(!defined($cell_2->[1]))){
	print ' and it represents the end of this lineage.';
    }
    print p(),a({-href=>"pedigree?name=$cell"},"Browse this cell's pedigree."),br,br;
} else {
    print img({-src=>'/images/cross1.gif'}),b(' PEDIGREE: '),
    " This is the parent of ",ObjectLink($cell_2->[0]),' and ',ObjectLink($cell_2->[1]),'.',br,br;
}
####embryo division time
my $embryo_division_time= $results->get('Embryo_division_time'=>1);

if (defined($embryo_division_time)){
    print img({-src=>'/images/cross1.gif'}),b(' TEMPORAL INFORMATION: '),
    b($cell),' appears ',$embryo_division_time,' minutes after the formation of the zygote.',br,br;
}else{
    print img({-src=>'/images/cross1.gif'}),b(' TEMPORAL INFORMATION: '),
    ' Presently we do not have temporal information about this particular cell.',br,br;
}

####paper

my @reference= $results->get('Reference'=>1);
if (@reference) {
    print img({-src=>'/images/cross1.gif'}), b(' CITATIONS: '),br();
    print start_blockquote(),start_dl();
    foreach my $reference (@reference) {
	my $acedb_paper = $reference->fetch;
	my $paper= $acedb_paper->get('Title'=>1);
	my $author= $acedb_paper->get('Author'=>1);
	my $year= $acedb_paper->get('Year'=>1);
	my $journal= $acedb_paper->get('Journal'=>1);
	print dt({-bgcolor=>'#ecf5ff'},b($author,' et Al. '));
	print dd(a({-href=>'/perl/ace/elegans/misc/paper?name='.$reference[0]},$paper),' ',$journal,' ',$year);
    }
    print end_dl(),end_blockquote();
}else{
    print img({-src=>'/images/cross1.gif'}),b(' CITATIONS: '),' None.',br,br;
}

##expression  pattern

my @expr_pattern= $results->get('Expr_pattern'=>1);

if (defined($expr_pattern[0])){
    my $num=@expr_pattern;
    print img({-src=>'/images/cross1.gif'}),b(' EXPRESSION PATTERNS: '),b($cell),
    ' has been associated with the following gene expression pattern(s): ';
    for (my $i=0; $i<$num; $i++){
	print a({href=>'/perl/ace/elegans/gene/expression?name='.$expr_pattern[$i]},$expr_pattern[$i]),'&nbsp;&nbsp;';
  }
}else{
    print img({-src=>'/images/cross1.gif'}),b(' EXPRESSION PATTERNS: '),' No associated expression Patterns';
}

print '<br><br><img src="/images/cross1.gif">',b(' ADDITIONAL INFORMATION: '),' Click on the pedigree on the right of the screen in order to jump to the corresponding Cell page.<br>';

Diagram($link_diagram,$cell) if defined $link_diagram;


#close main table print footer and stop web page

print_bottom;


# the program finish here
#-----------------------------------------------------------------
#down here starts the subroutines



sub Pedigree_map {
    my $results = shift;
    my $top_level = 0;
    my $cell = $results;
    
    my $cell_1= $results->get(Parent=>1);
    if (!$cell_1) {  #oops, at the top
	$cell = ($results->get('Daughter'=>1))[1];
	$cell_1 = $cell->Parent;
	$results = $cell->fetch;
	$top_level++;
    }
    my @cell_2= $results->get('Daughter'=>1);
    my $results1 = $cell_1->fetch if ref $cell_1;

    my @cell_3 = $results1->get('Daughter'=>1) if ref $results1;
    @cell_3 = reverse @cell_3 if $cell_3[0] eq $results;

    my $value3 = $cell_3[0]->fetch;
    my @cell_4= $value3->get('Daughter'=>1) if ref $value3;

    my $pic_link= 'make_pedigree.cgi?cell='.param('name');

    print img({-src=>$pic_link,
	       -border=>'0',
	       -align=>'right',
	       -usemap=>'#make_pedigree'});
  
    my @mapping;
  
  #1
    if (!$top_level && defined($cell_1)){
	push (@mapping,area({-shape=>'rect',
			     -coords=>'129,30,176,47',
			     -href=>Object2URL($cell_1),
			     -alt=>"Go to $cell_1"}),
	      )};

  #2
    if ($top_level) {
	push (@mapping,area({-shape=>'rect',
			     -coords=>'209,92,246,110',
			     -href=>Object2URL($results),
			     -alt=>"Go to $results"}),
	      )};
  
  #3
    if (defined($cell_3[0])){
	push (@mapping,area({'-shape'=>'rect',
			     '-coords'=>'53,92,102,109',
			     '-href'=>Object2URL($cell_3[0]),
			     '-alt'=>"Go to $cell_3[0]"}),
	      )};
    
  
  #4
    if (defined($cell_2[0])){
	push (@mapping,area({-shape=>'rect',
			     -coords=>'162,157,215,180',
			     -href=>Object2URL($cell_2[0]),
			     -alt=>"Go to $cell_2[0]"})
	 )};
  
  
  #5
    if (defined($cell_2[1])){
	push (@mapping,area({-shape=>'rect',
			     -coords=>'235,157,284,183',
			     -href=>Object2URL($cell_2[1]),
			     -alt=>"Go to $cell_2[1]"})
	 )};
  
  
  #6
    if (defined($cell_4[1])){
	push (@mapping,area({-shape=>'rect',
			     -coords=>'95,159,132,177',
			     -href=>Object2URL($cell_4[1]),
			     -alt=>"Go to $cell_4[1]"})
	 )};
  
  #7
    if (defined($cell_4[0])){
	push (@mapping,area({-shape=>'rect',
			     -coords=>'19,158,62,177',
			     -href=>Object2URL($cell_4[0]),
			     -alt=>"Go to $cell_4[0]"})
	    )};
  
    print Map({-name=>'make_pedigree'},@mapping);
    return ($cell_1,\@cell_2) unless $top_level;
    return (undef,[$cell_3[0],$cell]);
}


sub Error{
    my $cell = shift;
 
    print_top();  

#print img ({-src=>'no_pedigree.png', align=>'right', valign=>'top'});
    if (!$cell) {
	print h1('Cell Display');
	print_prompt_hunter();
    } else {
	print h1('Cell page - Result for ',$cell);
	print_prompt_hunter();
	print '<img src="/images/cross1.gif"> The cell named ',b($cell),' doesn\'t exists; please go back and type another cell',br,br;
    }
    print_bottom;
}

sub print_prompt_hunter {
  print
    start_form,
        p({-class=>'caption'},"Type in a cell name, such as ",cite('AB:')),
        p("Symbol: ",
        textfield(-name=>'name')),
      end_form;
}


sub Diagram{
    my ($link_diagram,$cell) = @_;
    my $diagram = Configuration->Neuron_diagrams;
  print p();
  print '
<table width="660" border cellspacing=0 cellpadding=4>
<th  bgcolor="#93cbf4">Diagram for ',$cell,'</th>
<tr>
<td align=center valign=center>';
    
  print img({-src=>"$diagram/$link_diagram"}),br;
  print '<b><font size=-2>&copy; 1988 by CSHL Press, do not use without the explicit consensus of the publisher</font></b>';
  print '</tr></td></table>';
}
