#!/usr/bin/perl
# file: dump_sequence.pl
# Dumps out a nucleotide file in FASTA format containing all
# the genomic segments.  Used for BLAST search script.
use Ace;

my $host = shift || 'localhost';
my $port = shift || 200005;

# connect to database
$db = Ace->connect(-port=>200005) || die "Couldn't open database";

# find all genomic sequences that contain DNA
@sequences = $db->fetch(Genome_Sequence => '*');
die "Couldn't get any genome sequences" unless @sequences;

# iterate through them
foreach my $s (@sequences) {
  
  # pull out interesting fields
  next unless my $dna = $s->DNA;
  $dna = $dna->right;
  my @genes = $s->Subsequence;  # predicted/actual genes

  # pull out identified or tentative genes
  my (%id,%tentative);
  foreach (@genes) {
    my $gene = $_->fetch;
    $id{$gene->Brief_identification(1)}++ if $gene->Brief_identification;
    $tentative{$gene->DB_remark(1)}++     if $gene->DB_remark;
    next if $gene->Brief_identification;

    next unless my $protein = $gene->Corresponding_protein;
    $protein = $protein->fetch;
    my $remark = $protein->Title(1) || 
      $protein->Other_name(1) ||
	$protein->DB_remark(1);
    $tentative{$remark}++ if $remark;
  }
  my @tentative = grep($_ && !$id{$_},keys %tentative);
  my @id        = keys %id;
  my ($gbk) = $s->AC_number;
  my ($map) = $s->Clone->Map if $s->Clone;
  my (@proteins) = 

  # print a fasta file
  print ">$s";
  print " /gb=$gbk" if $gbk;
  print " /cds=",scalar(@genes);
  foreach (@id) {
    tr/\n/ /;  # no newlines!
    print " /id=$_";
  }
  foreach (@tentative) {
    tr/\n/ /;  # no newlines!
    print " /tentative_id=$_";
  }
  print " /map=$map" if $map;
  print "\n";
  $dna =~ s/(.{80})/$1\n/g;
  print $dna,"\n";
}
