#!/usr/local/bin/perl
# file: dump_sequence.pl
# dumps out a FASTA file  suitable for BLAST searching against.
# contains all the known and predicted genes

use Ace;
use strict;

my $host = shift || 'stein.cshl.org';
my $port = shift || 200005;

# connect to database
my $db = Ace->connect(-host=>$host,
		      -port=>$port) || die "Couldn't open database";

# find all genomic sequences that are linked to a chromosome
my @sequences = $db->fetch('Sequence' => 'CHROMOSOME*');
die "Couldn't get any genome sequences" unless @sequences;

# recursively follow down to the gene level
for my $s (@sequences) {
  warn "processing genes on $s...\n";
  my @genes = dump_seqs($s,"$s",1);
}

sub dump_seqs {
  my ($seq,$chromosome,$offset,$len,%in) = @_;

  # If we're at a terminal sequence, dump it.
  my @subseq = sort {$a->right <=> $b->right} $seq->get('Subsequence');
  if ( !@subseq ) {
    dump_subsequence($seq,$chromosome,$offset,$offset+$len-1,%in);
    return;
  }

  foreach (@subseq) {
    my %info;
    my $s = $_->fetch;

    my $start  = $offset + $_->right-1;    # offset to beginning of sequence
    my $length = $_->right(2) - $_->right(1) + 1;

    $info{gbk} = $s->Database(3) if $s->Database;
    if (my $protein = $s->Corresponding_protein) {
      $protein = $protein->fetch;
      $info{remark} = $protein->Title(1) || $protein->DB_remark(1);
    }
    dump_seqs($s,$chromosome,$start,$length,%info,%in);
  }
}

sub dump_subsequence {
  my ($gene,$chromosome,$offset,$end,%info) = @_;
  return unless my $pep = $gene->asPeptide;
  return if $pep=~/\*/;

  print ">$gene";
  foreach (keys %info) {
    print " /$_=$info{$_}" if $info{$_};
  }

  $chromosome =~ s/^CHROMOSOME_//;
  print " /map=$chromosome:$offset-$end";

  $pep =~ s/^>.*//;
  unless ($pep =~ /\S/) {
    print "\n";
    next;
  }
  print $pep;
}
