#!/usr/local/bin/perl
# find the position of a sequence on the sequence map (in base pairs)

use strict;
use Ace;
use Getopt::Long;

my ($HOST,$PORT,$PATH,$DB);

GetOptions(
	   'host=s'  => \$HOST,
	   'port=i'  => \$PORT,
	   'path=s'  => \$PATH,
	  ) || die <<USAGE;
Usage: $0 [sequence list]
  Find genomic position of a list of sequences (in bp).

 Options:
          -host  <host>    database host
          -port  <port>    database port
          -path  <path>    local database path
USAGE

$HOST ||= 'stein.cshl.org';
$PORT ||= 200005;

# try to open database
$DB = Ace->connect(-path=>$PATH) if $PATH;
$DB = Ace->connect(-host=>$HOST,-port=>$PORT) unless $PATH;

while (my $seq = shift) {
  print_seq($seq);
}
exit 0;

sub print_seq {
  my $seq = shift;
  my ($start,$end,$ref) = find_position($seq);
  unless (defined $start) {
    print $seq,"\n";
    return;
  }
  print join("\t",$seq,$start,$end,$ref),"\n";
}

sub find_position {
  my $seq = shift;
  return undef unless my $s = $DB->fetch(Sequence=>$seq);
  my ($abs_offset,$length,$prev) = (1,0);
  
  for ($prev=$s, my $o=$s->Source; $o; $prev=$o,$o=$o->Source) {
    my @subs = $o->get('Subsequence');
    my ($seq) = grep $prev eq $_,@subs;
    $length ||= $seq->right(2) - $seq->right(1) + 1;
    $abs_offset += $seq->right-1;    # offset to beginning of sequence
  }
  
  return ($abs_offset,$abs_offset+$length-1,$prev);
}
