# ========= $NAME =========
# symbolic name of the database (defaults to name of file, lowercase)
$NAME = 'snp';
$ROOT = '/db/snp';
$CHROMROOT = '/chromtables';

# ========= $HOST =========
# name of the host to connect to
$HOST = 'localhost';

# ========= $PORT =========
# Port number to connect to
$PORT = 400000;

# ========= $STYLESHEET =========
# stylesheet to use
$STYLESHEET = '/stylesheets/snp2.css';

# ========= @SEARCHES =========
# search scripts available
# NOTE: the order is significant
@SEARCHES   = (
	       "$ROOT/map"     => 'Table Search',
	       "$ROOT/dynamap" => 'Graphical Search',
	       "$ROOT/index"   => 'Object Search',
	       "$ROOT/snpgb"   => 'Genbank Entry Search',
	       "searches/text/snp"    => 'Text Search',
	       "searches/query/snp"   => 'Acedb Query',
	       "$ROOT/snpftp"  => 'Bulk Download',
	      );

# Home page URL
@HOME      = ('http://snp.cshl.org' => '<b>TSC Home</b>');

# ========== An icon to use for "home" ==========
# leaving this undefined suppresses the generation of a "home" link
$HOME_ICON = '/icons/arrows/uarrw.gif';

# =========  An icon to use for searching =======
# leaving this undefined suppresses the generation of a "search" link
$SEARCH_ICON = '/icons/unknown.gif';


# displays to show
%DISPLAYS = ( 
	     'snp' =>   { 'url'   => "$ROOT/snp",
			 'label' => 'SNP Report',
			 'icon'  => '/icons/layout.gif'},
	     'clique' => { 'url'   => "$ROOT/clique",
			   'label' => 'Clique Report',
			    'icon'  => '/icons/layout.gif'},
	     'laboratory' =>  { 'url'   => "$ROOT/snplab",
				'label' => 'Laboratory Report',
				'icon'  => '/icons/layout.gif'},
	     'tree' =>  { 'url'   => "generic/tree/snp",
			  'label' => 'Tree Display',
			  'icon'  => '/icons/text.gif'},
	     'pic' =>  { 'url'   => "generic/pic/snp",
			  'label' => 'Graphic Display',
			  'icon'  => '/icons/image2.gif'},
	    );

%CLASSES = (
	    Snp    => ['snp'],
	    Clique => ['clique'],
	    Laboratory => ['laboratory'],
	    Default => ['tree','pic'],
);

# mapping from object type to URL.  Return empty list to fall through
# to default.
sub url_mapper {
    my ($display,$name,$class) = @_;
    $name = "?$1" if $class eq 'Model' && $name=~/^\#(.*)/;
    my $n = CGI::escape("$name");  # looks superfluous, but avoids Ace::Object name conversions errors
    my $c = CGI::escape("$class");
    return ('snp'        => "name=$n") if $class eq 'SNP';
    return ('clique'     => "name=$n") if $class eq 'Clique';
    return ('laboratory' => "name=$n") if $class eq 'Laboratory';
    return ('tree' => "name=$n&class=$c");
}

# Banner HTML
$BANNER = <<END;
<center><a href="http://snp.cshl.org">
<img border=0 src="http://snp.cshl.org/icons/logosmall.gif" width=486 height=36></a>
</center>
END

# Footer HTML
$FOOTER = <<END;
<img src="http://snp.cshl.org/icons/logobasso.gif">
<TABLE WIDTH="100%">
<TR>
    <TD  CLASS="small" VALIGN="TOP">
     <A HREF="mailto:snp-info\@snp.cshl.org">snp-info\@snp.cshl.org (database questions)</A>
    </TD>

    <TD  CLASS="small" ALIGN=RIGHT VALIGN=TOP>
    <A HREF="mailto:aholden\@earthlink.net">aholden\@earthlink.net (TSC questions)</A>
    </TD>
</TR>
</TABLE>
END

1;
