#!/usr/bin/perl 

use CGI qw/:standard :html3/;
use Chromosome;
use Ace;
use GD;
use constant PATH   => '~acedb/SNPdb';
use strict;

my $db = Ace->connect(-host=>'localhost', -port=>'400000') or
#my $db = Ace->connect(-path=>PATH) or
  die "Can't open database: ",Ace->error;

my $num = param('num') ? param('num') : 1;
my $chrom="Chrom$num";
if( param('contig') ) {
	my $contig = param('contig');
	contig_handler( $contig, $num, $db);
	exit;
}
my $start = param('start');
my $end = param('end');
my $old_start = param('old_start');
my $old_end = param('old_end');

my $c = $db->fetch(-class=>'CytogeneticMap',-name=>$chrom,-fill=>1) ||
  die "Can't fetch chromosome $chrom";

my $idiogram = Chromosome->new($c, $db);
$idiogram->restrict($start, $end) if $start;
my ($p_coords, $p_contig) = $idiogram->coords();
my $p_image_map = get_extremes($p_coords);
if( param('image') eq 'yes') {
	print header($GD::VERSION > 1.19 ? 'image/png' : 'image/gif');
	print $idiogram->draw(1);
} else {
	print header('text/html'),
	start_html(-style=>{-src=>'/stylesheets/snp2.css'}),
	start_form(-action=>"/db/snp/chrom.pl",
			   -method=>"GET"),
	hidden("num", $num),
	submit(-name=>"Zoom Out"),
	font({-face=>"Arial,Helvetica,sans-serif", -size=>"-1"},
	"&nbsp;&nbsp;&nbsp;&nbsp;Click on the chromosome to zoom in, or on a contig to see its parts."), br(),
	img({-src=>"/db/snp/chrom.pl?image=yes&num=$num&start=$start&end=$end",
	     -ismap=>"",
	     -border=>0,
	     -usemap=>"#bands"}),
	end_form();
	print_image_map( $p_coords, $p_image_map, $p_contig, $num );
	print end_html();
}

############################## sub get_extremes
sub get_extremes {
	my $p_coords = shift;
	my %coords = %{$p_coords};
	my %map_data;
	my @bands= sort {$coords{$a}->[0] <=> $coords{$b}->[0]} keys %coords;
	for (@bands) {
		my $top = $coords{$_}->[0];
		my $bottom = $coords{$_}->[1];
	}
	for(  my $i = 0; $i <= $#bands; $i++ ) {
		my $band = $bands[$i];
		my @range = ();
		push @range, $band;
		my $forward = $i;
		my $backward = $i;
		my $sign = '1';
		while( ($coords{$range[$#range]}->[1] - $coords{$range[0]}->[0]) < (0.5 * ($coords{$bands[$#bands]}->[1] - $coords{$bands[0]}->[0]))) {
			if( $sign ==1) {
				if ($forward > $#bands) {
					last if $backward < 0;
					$sign = -1 * $sign;
					next;
				} else {
					$forward++;
					$sign = -1 * $sign;
					next if $forward > $#bands;
					push @range, $bands[$forward];
				}
			} else {
				if ($backward < 0) {
					last if $forward > $#bands;
					$sign = -1 * $sign;
					next;
				} else {
					$backward--;
					$sign = -1 * $sign;
					next if $backward < 0;
					unshift @range, $bands[$backward];
				}
			}
		}
		push @{$map_data{$band}}, ($range[0],$range[$#range]);
	}
	return \%map_data;
}

############################## sub print_image_map
sub print_image_map {
	my ($p_coords, $p_image_map, $p_contig, $num) = @_;
	my %coords = %{$p_coords};
	my %image_map = %{$p_image_map};
	my %contig = %{$p_contig};
	print "\n<map name=\"bands\">\n";
	for( keys %coords ) {
		my $left = $coords{$_}->[2];
		my $top = $coords{$_}->[0];
		my $right = $coords{$_}->[3];
		my $bottom = $coords{$_}->[1];
		my $band1 = $image_map{$_}->[0];
		my $band2 = $image_map{$_}->[1];
		print qq!<area shape=rect coords="$left,$top,$right,$bottom"\n!;
		print qq!href="/db/snp/chrom.pl?num=$num&start=$band1&end=$band2" target="picture">\n!;
	}

	for ( keys %contig ) {
		my $contig = $_;
		my $top = $contig{$contig}->{y_string};
		my $left = $contig{$contig}->{x_string};
		my $bottom = $top + 8;
		my $right = $left + 50;
		my $line_top = $contig{$contig}->{y1};
		my $line_bottom = $contig{$contig}->{y2};
		my $line_x = $contig{$contig}->{x};
		my $line_left = $line_x - 1;
		my $line_right = $line_x + 1;
		print qq!<area shape=rect coords="$left,$top,$right,$bottom"\n!;
		print qq!href="/db/snp/chrom.pl?contig=$contig&num=$num" target="table">\n!;
		print qq!<area shape=rect coords="$line_left,$line_top,$line_right,$line_bottom"\n!;
		print qq!href="/db/snp/chrom.pl?contig=$contig&num=$num" target="table">\n!;
	}
	print "</map>\n",
}

######################################## sub contig_handler
sub contig_handler {
	my ($contig, $num, $db) = @_;
	print header('text/html'),
	start_html(-title=>"Contig $contig",
			   -style=>{-src=>'/stylesheets/snp2.css'}
			  ),
	h4("Chromosome: $num"),
	"\n",
	h4("Contig: $contig"),
	"\n";
	print "<table border = 1>\n",
	"<tr><th>SNP</th><th>Clique</th><th>Accession</th></tr>\n";

	my $link = $db->fetch(-class=>'Link', -name=>$contig);
	my @acc = $link->SC_Sequence;
	for my $acc (@acc) {
		if($acc->SC_Clique) {
			my @clique = $acc->SC_Clique;
			for my $clique (@clique) {
				my @snp = $clique->SC_SNP;
				for my $snp (@snp) {
					print qq!<tr><td><a href="/db/snp/snp?name=$snp" target="_top">$snp</a></td><td>$clique</td><td>$acc</td></tr>\n!;
				}
			}
		} else {
			print "<tr><td>&nbsp;</td><td>&nbsp;</td><td>$acc</td></tr>\n";
		}
	}
	print "</table>\n";
	print end_html();
}

#	my $link = $db->fetch(-class=>'Link', -name=>$contig);
#	my @acc = $link->SC_Sequence;
#	for my $acc (@acc) {
#		my $clique = $acc->SC_Clique if $acc->SC_Clique;
#		if ($clique) {
#			my @snp = $clique->SC_SNP if $clique and $clique->SC_SNP;
#			for my $snp (@snp) {
#				print "Accession $acc.&nbsp;&nbsp;Clique $clique.&nbsp;&nbsp;Snp $snp\n";
##				print br();
#			my $rowhead = $snp;
#			my @data = ($clique, $acc);
#			push( @rows, th($rowhead).td(\@data) );
#			}
#		} else {
#			my $rowhead = '&nbsp;';
#			my @data = (&nbsp, $acc);
#			push( @rows, th($rowhead).td(\@data) );
#			print "Accession $acc\n",
#			br();
#		}
#	}
#	print table({-border=>''},
#				TR([th(\@col_head),@rows])
#			   ),
#	"</font>",	  
#	end_html();
#}
	
	
	
