package Contig;
use Ace;
use GD;
use strict;
use constant V_MARGIN => 0;
use constant H_MARGIN => 10;
use constant H_RULER => 150;  # horizontal position of the ruler
use constant H_PARTS => 120;  # horizontal position of the parts
use constant BUMP    => 5;  # how much contigs "bump" each other to the right
use constant STRING_START => 30; # positions string right of contig bars
use constant Y_ADJUST => -3; # adjust relative positions of contigs and strings
use constant FH_ADJUST => -2; # adjust how tightly strings can be stacked
use constant TICKS  => 20;
use constant TICKWIDTH => 6;
use constant PATH   => '~acedb/SNPdb';

sub contig {
	my $p_parsed_bands = shift;
	my @parsed_bands = @{$p_parsed_bands};
	my $start_index = shift;
	my $end_index = shift;
	my $chrom = shift;
	my $db = shift;
	my $default_width = shift;
	my $default_height  = shift;
	my $top_marg = shift;
	die "Can't extract chromosome number from $chrom\n" unless $chrom =~ /([\dXY]+)/i;
	$chrom = "Chr$1";
	
	my $start_band = $parsed_bands[$start_index]->[0];
	my $end_band = $parsed_bands[$end_index]->[0];
	
	my $c = $db->fetch(-class=>'Chromosome',-name=>$chrom,-fill=>1) || die "Can't fetch chromosome $chrom";
	
	# allocate some colors
	my @parts = sort {$a->right <=> $b->right} $c->SC_Link;
	
	my %bands = ();
 	
 	for(@parsed_bands) {
 		my $name = $_->[0];
 		$bands{$name}->[0] = $_->[1];
 		$bands{$name}->[1] = $_->[2];
 	}

	my $bp_start = undef;
	my $bp_end = undef;
	my $conversion =  $parts[$#parts]->right(2) / $parsed_bands[$#parsed_bands]->[2];

	$bp_start = int( $bands{$start_band}->[0] * $conversion ) if $start_band;
	$bp_end = int( $bands{$end_band}->[1] * $conversion ) if $end_band;
	@parts = grep {$_->right >= $bp_start} @parts if $bp_start;
	@parts = grep {$_->right(2) <= $bp_end} @parts if $bp_end;
	die "Nothing defined in selected range" unless @parts;

	# @parts now holds sorted list of ends
	my $start = $parts[0]->right;
	my $end = $parts[$#parts]->right(2);
	
	$bp_start ||= $start unless defined $bp_start;
	$bp_end   ||= $end unless defined $bp_end;

	# round to two places
	my $exponent = int log10($end) - 2;
	my $Max = int($bp_end/(10**$exponent))* 10**$exponent;
	my $Min = int(0.99 + $bp_start/(10**$exponent)) * 10**$exponent;
	my $PixelsPerBP = ($default_height - V_MARGIN*2)/($Max - $Min);
	
	my ($x,$y);
	$x = H_MARGIN + H_RULER;

	my $fh = gdTinyFont->height;
	my $fw = gdTinyFont->width;

	# -------------- MAIN RULER LINE ---------------
	# want TICKS ticks on the ruler
#	my $units = int(($Max-$Min)/TICKS);
#	$x = int(H_MARGIN + H_RULER - TICKWIDTH/2);
#	if($draw) {
#		$pic->string(gdSmallFont,H_RULER-10,V_MARGIN,"Mb",$black);
#		$pic->line($x,b2p(0,$Min,$PixelsPerBP),$x,b2p($Max,$Min,$PixelsPerBP),$black);
#		for (my $i = $Min; $i <= $Max; $i += $units) {
#			$y = b2p($i,$Min,$PixelsPerBP);
#			$pic->line($x,$y,$x + TICKWIDTH,$y,$black);
#			my $mb = sprintf("%2.1f",$i/1E6);
#			my $offset = length($mb) * $fw;
#			$pic->string(gdTinyFont,$x - $offset,$y-$fh,$mb,$black);
#		}
#	}
	# -------------- CONTIGS ---------------
	# make the pen a little bigger
# this doesn't seem to do anything...
#	my $brush = GD::Image->new(2,2);
#	$brush->colorAllocate(255,255,255);
#	my $blue = $brush->colorAllocate(0,0,255);
#	$brush->fill(1,1,$blue);
	
	my $offset = 0;
	my $string_offset = 0;
	my $obstruction = $top_marg;
	my $string_obstruction = $top_marg + Y_ADJUST;
	my $offset_count = 0;  # counts parts moving away from baseline
	$x = H_PARTS;
	#------------ DRAW CONTIGS AND NAMES ------------
	my %CONTIGS;
	my $print = 0;
	my $max_ystring;
	open LOG, ">>position.txt"; # or die "can't write\n";
	for my $part (@parts) {
		my $display_string = 1; #should Chromosome draw contig string?
		my $display_line = 1;   #should Chromosome draw contig line?
#		$print= 1 if $part =~ /002209/; # for testing
#		$print = 0 if $part =~ /002166/; # for testing

		my $y1 = $top_marg + b2p($part->right,$Min,$PixelsPerBP);
		my $y2 = $top_marg + b2p($part->right(2),$Min,$PixelsPerBP);
		next if $y2 == $top_marg; #eliminate links with no physpos
		$offset_count++;

		if ($y1 > $obstruction) {
			$offset = 0;
			$obstruction = $y2;
		}

		my $y1_string = $y1 + Y_ADJUST; 
		my $y2_string = $y1_string + $fh + FH_ADJUST;

		if ($y1_string >= $string_obstruction) {
			$string_offset = 0;
			$string_obstruction = $y2_string;
			$offset_count = 0;
		}
		my $x_string = $x + STRING_START + $string_offset;

		# bump string obstruction if bottom of current string is below current obstruction
		# and if the string is more or less meant to be displayed ($offset_count < 8)
		$string_obstruction = $y2_string if $y2_string > $string_obstruction and $offset_count < 8;

		print LOG "part <$part>,  x_str <$x_string>, y1_str <$y1_string>, y2_str <$y2_string>, offset <$string_offset>, count <$offset_count>, obstruction <$string_obstruction>\n" if $print;

		#don't display if string outside frame
		$display_string = 0 if $x_string + 50 > $default_width;

		#don't display contig line if  beyond start of strings. -5 is fudge factor
		$display_line = 0 if ($x + $offset - H_PARTS) > (STRING_START - 5); 

		@{$CONTIGS{$part}}{qw/x y1 y2 x_string y_string display_string display_line/} = ($x+$offset, $y1, $y2, $x_string, $y1_string, $display_string, $display_line); #gets passed a "display" variable
		$offset += BUMP;
		$string_offset += 50;
	}
	
	print LOG "\n";
	return \%CONTIGS
}

sub log10 { return log($_[0])/log(10); }
sub b2p   {
	my $val = shift;
	my $Min = shift;
	my$PixelsPerBP = shift;
	return V_MARGIN + int(($val-$Min) * $PixelsPerBP + 0.5); }

1;
